function d = dsi(reference, distorted, varargin)
%
% DisSimilarity Index is a full-reference metric for image visual quality assessment
%
% Inputs
% ---------
%
% Required:
%
% reference - grayscale reference image
% distorted - grayscale distorted image
%
% Optional:
%
% 'bs'  - patch (block) size (default value: 5)
% 'hsz' - half of size of search zone of similar patches (default value: 9)
% 'zp'  - size of area (default value:2) around of each patch which will be 
%         excluded from searching to provide robustness to spatially correlated noise. 
%
% Output:
% ---------
%
%  d - value of calculated DisSimilarity Index
%
% Examples of usage
% -----------------
%
% d = dsi(a, b);
% d = dsi(a, b, 'bs',7);
% d = dsi(a, b, 'hsz',10, 'zp',0);
% d = dsi(a, b, 'bs',3, 'hsz',7, 'zp',1);
%
% References:
% -----------
%
% K. Egiazarian, M. Ponomarenko, V. Lukin, O. Ieremeiev, 
% Statistical evaluation of visual quality metrics for image denoising,
% Submitted to ICASSP 2018.

bs = 5; hsz = 9; zp = 2;  % default values

for i = 1:fix((nargin-2)/2)
  name = varargin{i*2-1};
  value = varargin{i*2};
  switch lower(name)
    case 'bs'
      bs = value;
    case 'hsz'
      hsz = value;
    case 'zp'
      zp = value;
    otherwise
      disp(['Unknown parameter: ' name '!']);
      d = 0;
      return; 
  end
end

d1 = mapdissim(reference,bs,hsz,zp);
d2 = mapdissim(distorted,bs,hsz,zp);

d1=d1.^0.5; d2=d2.^0.5;

d = max(0,abs(d1-d2)-d2/4.5).^2;
d = mean(d(:));

end
